@@ -23,24 +23,19 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
+#include "id3v2frame.h"
+
+#include <array>
 #include <bitset>
 
-#include <tdebug.h>
-#include <tstringlist.h>
-#include <tzlib.h>
-
-#include "id3v2tag.h"
-#include "id3v2frame.h"
-#include "id3v2synchdata.h"
-
+#include "tdebug.h"
+#include "tstringlist.h"
+#include "tzlib.h"
 #include "tpropertymap.h"
+#include "id3v2tag.h"
+#include "id3v2synchdata.h"
 #include "frames/textidentificationframe.h"
-#include "frames/urllinkframe.h"
-#include "frames/unsynchronizedlyricsframe.h"
-#include "frames/commentsframe.h"
-#include "frames/uniquefileidentifierframe.h"
 #include "frames/unknownframe.h"
-#include "frames/podcastframe.h"
 
 using namespace TagLib;
 using namespace ID3v2;
@@ -48,16 +43,16 @@ using namespace ID3v2;
 class Frame::FramePrivate
 {
 public:
-  FramePrivate() :
-    header(0)
-    {}
-
+  FramePrivate() = default;
   ~FramePrivate()
   {
     delete header;
   }
 
-  Frame::Header *header;
+  FramePrivate(const FramePrivate &) = delete;
+  FramePrivate &operator=(const FramePrivate &) = delete;
+
+  Frame::Header *header { nullptr };
 };
 
 namespace
@@ -67,12 +62,8 @@ namespace
     if(frameID.size() != 4)
       return false;
 
-    for(ByteVector::ConstIterator it = frameID.begin(); it != frameID.end(); it++) {
-      if( (*it < 'A' || *it > 'Z') && (*it < '0' || *it > '9') ) {
-        return false;
-      }
-    }
-    return true;
+    return std::none_of(frameID.begin(), frameID.end(),
+      [](auto c) { return (c < 'A' || c > 'Z') && (c < '0' || c > '9'); });
   }
 }  // namespace
 
@@ -80,16 +71,6 @@ namespace
 // static methods
 ////////////////////////////////////////////////////////////////////////////////
 
-unsigned int Frame::headerSize()
-{
-  return Header::size();
-}
-
-unsigned int Frame::headerSize(unsigned int version)
-{
-  return Header::size(version);
-}
-
 ByteVector Frame::textDelimiter(String::Type t)
 {
   if(t == String::UTF16 || t == String::UTF16BE || t == String::UTF16LE)
@@ -106,60 +87,12 @@ const String Frame::urlPrefix("URL:");
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-Frame *Frame::createTextualFrame(const String &key, const StringList &values) //static
+unsigned int Frame::headerSize() const
 {
-  // check if the key is contained in the key<=>frameID mapping
-  ByteVector frameID = keyToFrameID(key);
-  if(!frameID.isEmpty()) {
-    // Apple proprietary WFED (Podcast URL), MVNM (Movement Name), MVIN (Movement Number), GRP1 (Grouping) are in fact text frames.
-    if(frameID[0] == 'T' || frameID == "WFED" || frameID == "MVNM" || frameID == "MVIN" || frameID == "GRP1"){ // text frame
-      TextIdentificationFrame *frame = new TextIdentificationFrame(frameID, String::UTF8);
-      frame->setText(values);
-      return frame;
-    } if((frameID[0] == 'W') && (values.size() == 1)){  // URL frame (not WXXX); support only one value
-        UrlLinkFrame* frame = new UrlLinkFrame(frameID);
-        frame->setUrl(values.front());
-        return frame;
-    } if(frameID == "PCST") {
-      return new PodcastFrame();
-    }
-  }
-  if(key == "MUSICBRAINZ_TRACKID" && values.size() == 1) {
-    UniqueFileIdentifierFrame *frame = new UniqueFileIdentifierFrame("http://musicbrainz.org", values.front().data(String::UTF8));
-    return frame;
-  }
-  // now we check if it's one of the "special" cases:
-  // -LYRICS: depending on the number of values, use USLT or TXXX (with description=LYRICS)
-  if((key == "LYRICS" || key.startsWith(lyricsPrefix)) && values.size() == 1){
-    UnsynchronizedLyricsFrame *frame = new UnsynchronizedLyricsFrame(String::UTF8);
-    frame->setDescription(key == "LYRICS" ? key : key.substr(lyricsPrefix.size()));
-    frame->setText(values.front());
-    return frame;
-  }
-  // -URL: depending on the number of values, use WXXX or TXXX (with description=URL)
-  if((key == "URL" || key.startsWith(urlPrefix)) && values.size() == 1){
-    UserUrlLinkFrame *frame = new UserUrlLinkFrame(String::UTF8);
-    frame->setDescription(key == "URL" ? key : key.substr(urlPrefix.size()));
-    frame->setUrl(values.front());
-    return frame;
-  }
-  // -COMMENT: depending on the number of values, use COMM or TXXX (with description=COMMENT)
-  if((key == "COMMENT" || key.startsWith(commentPrefix)) && values.size() == 1){
-    CommentsFrame *frame = new CommentsFrame(String::UTF8);
-    if (key != "COMMENT"){
-      frame->setDescription(key.substr(commentPrefix.size()));
-    }
-    frame->setText(values.front());
-    return frame;
-  }
-  // if non of the above cases apply, we use a TXXX frame with the key as description
-  return new UserTextIdentificationFrame(keyToTXXX(key), values, String::UTF8);
+  return d->header->size();
 }
 
-Frame::~Frame()
-{
-  delete d;
-}
+Frame::~Frame() = default;
 
 ByteVector Frame::frameID() const
 {
@@ -182,7 +115,11 @@ void Frame::setData(const ByteVector &data)
 
 void Frame::setText(const String &)
 {
+}
 
+StringList Frame::toStringList() const
+{
+  return toString();
 }
 
 ByteVector Frame::render() const
@@ -194,27 +131,141 @@ ByteVector Frame::render() const
   return headerData + fieldData;
 }
 
+Frame::Header *Frame::header() const
+{
+  return d->header;
+}
+
+namespace
+{
+  constexpr std::array frameTranslation {
+    // Text information frames
+    std::pair("TALB", "ALBUM"),
+    std::pair("TBPM", "BPM"),
+    std::pair("TCOM", "COMPOSER"),
+    std::pair("TCON", "GENRE"),
+    std::pair("TCOP", "COPYRIGHT"),
+    std::pair("TDEN", "ENCODINGTIME"),
+    std::pair("TDLY", "PLAYLISTDELAY"),
+    std::pair("TDOR", "ORIGINALDATE"),
+    std::pair("TDRC", "DATE"),
+    // std::pair("TRDA", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
+    // std::pair("TDAT", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
+    // std::pair("TYER", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
+    // std::pair("TIME", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
+    std::pair("TDRL", "RELEASEDATE"),
+    std::pair("TDTG", "TAGGINGDATE"),
+    std::pair("TENC", "ENCODEDBY"),
+    std::pair("TEXT", "LYRICIST"),
+    std::pair("TFLT", "FILETYPE"),
+    // std::pair("TIPL", "INVOLVEDPEOPLE"), handled separately
+    std::pair("TIT1", "WORK"), // 'Work' in iTunes
+    std::pair("TIT2", "TITLE"),
+    std::pair("TIT3", "SUBTITLE"),
+    std::pair("TKEY", "INITIALKEY"),
+    std::pair("TLAN", "LANGUAGE"),
+    std::pair("TLEN", "LENGTH"),
+    // std::pair("TMCL", "MUSICIANCREDITS"), handled separately
+    std::pair("TMED", "MEDIA"),
+    std::pair("TMOO", "MOOD"),
+    std::pair("TOAL", "ORIGINALALBUM"),
+    std::pair("TOFN", "ORIGINALFILENAME"),
+    std::pair("TOLY", "ORIGINALLYRICIST"),
+    std::pair("TOPE", "ORIGINALARTIST"),
+    std::pair("TOWN", "OWNER"),
+    std::pair("TPE1", "ARTIST"),
+    std::pair("TPE2", "ALBUMARTIST"), // id3's spec says 'PERFORMER', but most programs use 'ALBUMARTIST'
+    std::pair("TPE3", "CONDUCTOR"),
+    std::pair("TPE4", "REMIXER"),     // could also be ARRANGER
+    std::pair("TPOS", "DISCNUMBER"),
+    std::pair("TPRO", "PRODUCEDNOTICE"),
+    std::pair("TPUB", "LABEL"),
+    std::pair("TRCK", "TRACKNUMBER"),
+    std::pair("TRSN", "RADIOSTATION"),
+    std::pair("TRSO", "RADIOSTATIONOWNER"),
+    std::pair("TSOA", "ALBUMSORT"),
+    std::pair("TSOC", "COMPOSERSORT"),
+    std::pair("TSOP", "ARTISTSORT"),
+    std::pair("TSOT", "TITLESORT"),
+    std::pair("TSO2", "ALBUMARTISTSORT"), // non-standard, used by iTunes
+    std::pair("TSRC", "ISRC"),
+    std::pair("TSSE", "ENCODING"),
+    std::pair("TSST", "DISCSUBTITLE"),
+    // URL frames
+    std::pair("WCOP", "COPYRIGHTURL"),
+    std::pair("WOAF", "FILEWEBPAGE"),
+    std::pair("WOAR", "ARTISTWEBPAGE"),
+    std::pair("WOAS", "AUDIOSOURCEWEBPAGE"),
+    std::pair("WORS", "RADIOSTATIONWEBPAGE"),
+    std::pair("WPAY", "PAYMENTWEBPAGE"),
+    std::pair("WPUB", "PUBLISHERWEBPAGE"),
+    // std::pair("WXXX", "URL"), handled specially
+    // Other frames
+    std::pair("COMM", "COMMENT"),
+    // std::pair("USLT", "LYRICS"), handled specially
+    // Apple iTunes proprietary frames
+    std::pair("PCST", "PODCAST"),
+    std::pair("TCAT", "PODCASTCATEGORY"),
+    std::pair("TDES", "PODCASTDESC"),
+    std::pair("TGID", "PODCASTID"),
+    std::pair("WFED", "PODCASTURL"),
+    std::pair("MVNM", "MOVEMENTNAME"),
+    std::pair("MVIN", "MOVEMENTNUMBER"),
+    std::pair("GRP1", "GROUPING"),
+    std::pair("TCMP", "COMPILATION"),
+  };
+
+  // list of deprecated frames and their successors
+  constexpr std::array deprecatedFrames {
+    std::pair("TRDA", "TDRC"), // 2.3 -> 2.4 (http://en.wikipedia.org/wiki/ID3)
+    std::pair("TDAT", "TDRC"), // 2.3 -> 2.4
+    std::pair("TYER", "TDRC"), // 2.3 -> 2.4
+    std::pair("TIME", "TDRC"), // 2.3 -> 2.4
+  };
+}  // namespace
+
+String Frame::frameIDToKey(const ByteVector &id)
+{
+  ByteVector id24 = id;
+  for(const auto &[o, t] : deprecatedFrames) {
+    if(id24 == o) {
+      id24 = t;
+      break;
+    }
+  }
+  for(const auto &[o, t] : frameTranslation) {
+    if(id24 == o)
+      return t;
+  }
+  return String();
+}
+
+ByteVector Frame::keyToFrameID(const String &s)
+{
+  const String key = s.upper();
+  for(const auto &[o, t] : frameTranslation) {
+    if(key == t)
+      return o;
+  }
+  return ByteVector();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // protected members
 ////////////////////////////////////////////////////////////////////////////////
 
 Frame::Frame(const ByteVector &data) :
-  d(new FramePrivate())
+  d(std::make_unique<FramePrivate>())
 {
   d->header = new Header(data);
 }
 
 Frame::Frame(Header *h) :
-  d(new FramePrivate())
+  d(std::make_unique<FramePrivate>())
 {
   d->header = h;
 }
 
-Frame::Header *Frame::header() const
-{
-  return d->header;
-}
-
 void Frame::setHeader(Header *h, bool deleteCurrent)
 {
   if(deleteCurrent)
@@ -235,7 +286,7 @@ void Frame::parse(const ByteVector &data)
 
 ByteVector Frame::fieldData(const ByteVector &frameData) const
 {
-  unsigned int headerSize = Header::size(d->header->version());
+  unsigned int headerSize = d->header->size();
 
   unsigned int frameDataOffset = headerSize;
   unsigned int frameDataLength = size();
@@ -244,6 +295,13 @@ ByteVector Frame::fieldData(const ByteVector &frameData) const
     frameDataLength = SynchData::toUInt(frameData.mid(headerSize, 4));
     frameDataOffset += 4;
   }
+  if(frameData.size() >= headerSize &&
+     frameDataOffset + frameDataLength > frameData.size()) {
+    // The first check is needed because some "dual purpose" frame constructors
+    // call this method with only the frame ID, i.e. without a complete header.
+    debug("Invalid frame data length");
+    return ByteVector();
+  }
 
   if(zlib::isAvailable() && d->header->compression() && !d->header->encryption()) {
     if(frameData.size() <= frameDataOffset) {
@@ -287,22 +345,17 @@ String Frame::readStringField(const ByteVector &data, String::Type encoding, int
   return str;
 }
 
-String::Type Frame::checkEncoding(const StringList &fields, String::Type encoding) // static
+String::Type Frame::checkTextEncoding(const StringList &fields, String::Type encoding) const
 {
-  return checkEncoding(fields, encoding, 4);
-}
-
-String::Type Frame::checkEncoding(const StringList &fields, String::Type encoding, unsigned int version) // static
-{
-  if((encoding == String::UTF8 || encoding == String::UTF16BE) && version != 4)
+  if((encoding == String::UTF8 || encoding == String::UTF16BE) && header()->version() != 4)
     return String::UTF16;
 
   if(encoding != String::Latin1)
     return encoding;
 
-  for(StringList::ConstIterator it = fields.begin(); it != fields.end(); ++it) {
-    if(!(*it).isLatin1()) {
-      if(version == 4) {
+  for(const auto &field : fields) {
+    if(!field.isLatin1()) {
+      if(header()->version() == 4) {
         debug("Frame::checkEncoding() -- Rendering using UTF8.");
         return String::UTF8;
       }
@@ -314,190 +367,16 @@ String::Type Frame::checkEncoding(const StringList &fields, String::Type encodin
   return String::Latin1;
 }
 
-String::Type Frame::checkTextEncoding(const StringList &fields, String::Type encoding) const
-{
-  return checkEncoding(fields, encoding, header()->version());
-}
-
-namespace
-{
-  const std::pair<const char *, const char *> frameTranslation[] = {
-    // Text information frames
-    std::make_pair("TALB", "ALBUM"),
-    std::make_pair("TBPM", "BPM"),
-    std::make_pair("TCOM", "COMPOSER"),
-    std::make_pair("TCON", "GENRE"),
-    std::make_pair("TCOP", "COPYRIGHT"),
-    std::make_pair("TDEN", "ENCODINGTIME"),
-    std::make_pair("TDLY", "PLAYLISTDELAY"),
-    std::make_pair("TDOR", "ORIGINALDATE"),
-    std::make_pair("TDRC", "DATE"),
-    // std::make_pair("TRDA", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
-    // std::make_pair("TDAT", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
-    // std::make_pair("TYER", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
-    // std::make_pair("TIME", "DATE"), // id3 v2.3, replaced by TDRC in v2.4
-    std::make_pair("TDRL", "RELEASEDATE"),
-    std::make_pair("TDTG", "TAGGINGDATE"),
-    std::make_pair("TENC", "ENCODEDBY"),
-    std::make_pair("TEXT", "LYRICIST"),
-    std::make_pair("TFLT", "FILETYPE"),
-    //std::make_pair("TIPL", "INVOLVEDPEOPLE"), handled separately
-    std::make_pair("TIT1", "WORK"), // 'Work' in iTunes
-    std::make_pair("TIT2", "TITLE"),
-    std::make_pair("TIT3", "SUBTITLE"),
-    std::make_pair("TKEY", "INITIALKEY"),
-    std::make_pair("TLAN", "LANGUAGE"),
-    std::make_pair("TLEN", "LENGTH"),
-    //std::make_pair("TMCL", "MUSICIANCREDITS"), handled separately
-    std::make_pair("TMED", "MEDIA"),
-    std::make_pair("TMOO", "MOOD"),
-    std::make_pair("TOAL", "ORIGINALALBUM"),
-    std::make_pair("TOFN", "ORIGINALFILENAME"),
-    std::make_pair("TOLY", "ORIGINALLYRICIST"),
-    std::make_pair("TOPE", "ORIGINALARTIST"),
-    std::make_pair("TOWN", "OWNER"),
-    std::make_pair("TPE1", "ARTIST"),
-    std::make_pair("TPE2", "ALBUMARTIST"), // id3's spec says 'PERFORMER', but most programs use 'ALBUMARTIST'
-    std::make_pair("TPE3", "CONDUCTOR"),
-    std::make_pair("TPE4", "REMIXER"), // could also be ARRANGER
-    std::make_pair("TPOS", "DISCNUMBER"),
-    std::make_pair("TPRO", "PRODUCEDNOTICE"),
-    std::make_pair("TPUB", "LABEL"),
-    std::make_pair("TRCK", "TRACKNUMBER"),
-    std::make_pair("TRSN", "RADIOSTATION"),
-    std::make_pair("TRSO", "RADIOSTATIONOWNER"),
-    std::make_pair("TSOA", "ALBUMSORT"),
-    std::make_pair("TSOC", "COMPOSERSORT"),
-    std::make_pair("TSOP", "ARTISTSORT"),
-    std::make_pair("TSOT", "TITLESORT"),
-    std::make_pair("TSO2", "ALBUMARTISTSORT"), // non-standard, used by iTunes
-    std::make_pair("TSRC", "ISRC"),
-    std::make_pair("TSSE", "ENCODING"),
-    // URL frames
-    std::make_pair("WCOP", "COPYRIGHTURL"),
-    std::make_pair("WOAF", "FILEWEBPAGE"),
-    std::make_pair("WOAR", "ARTISTWEBPAGE"),
-    std::make_pair("WOAS", "AUDIOSOURCEWEBPAGE"),
-    std::make_pair("WORS", "RADIOSTATIONWEBPAGE"),
-    std::make_pair("WPAY", "PAYMENTWEBPAGE"),
-    std::make_pair("WPUB", "PUBLISHERWEBPAGE"),
-    //std::make_pair("WXXX", "URL"), handled specially
-    // Other frames
-    std::make_pair("COMM", "COMMENT"),
-    //std::make_pair("USLT", "LYRICS"), handled specially
-    // Apple iTunes proprietary frames
-    std::make_pair("PCST", "PODCAST"),
-    std::make_pair("TCAT", "PODCASTCATEGORY"),
-    std::make_pair("TDES", "PODCASTDESC"),
-    std::make_pair("TGID", "PODCASTID"),
-    std::make_pair("WFED", "PODCASTURL"),
-    std::make_pair("MVNM", "MOVEMENTNAME"),
-    std::make_pair("MVIN", "MOVEMENTNUMBER"),
-    std::make_pair("GRP1", "GROUPING"),
-    std::make_pair("TCMP", "COMPILATION"),
-  };
-  const size_t frameTranslationSize = sizeof(frameTranslation) / sizeof(frameTranslation[0]);
-
-  const std::pair<const char *, const char *> txxxFrameTranslation[] = {
-    std::make_pair("MUSICBRAINZ ALBUM ID",         "MUSICBRAINZ_ALBUMID"),
-    std::make_pair("MUSICBRAINZ ARTIST ID",        "MUSICBRAINZ_ARTISTID"),
-    std::make_pair("MUSICBRAINZ ALBUM ARTIST ID",  "MUSICBRAINZ_ALBUMARTISTID"),
-    std::make_pair("MUSICBRAINZ ALBUM RELEASE COUNTRY", "RELEASECOUNTRY"),
-    std::make_pair("MUSICBRAINZ ALBUM STATUS",     "RELEASESTATUS"),
-    std::make_pair("MUSICBRAINZ ALBUM TYPE",       "RELEASETYPE"),
-    std::make_pair("MUSICBRAINZ RELEASE GROUP ID", "MUSICBRAINZ_RELEASEGROUPID"),
-    std::make_pair("MUSICBRAINZ RELEASE TRACK ID", "MUSICBRAINZ_RELEASETRACKID"),
-    std::make_pair("MUSICBRAINZ WORK ID",          "MUSICBRAINZ_WORKID"),
-    std::make_pair("ACOUSTID ID",                  "ACOUSTID_ID"),
-    std::make_pair("ACOUSTID FINGERPRINT",         "ACOUSTID_FINGERPRINT"),
-    std::make_pair("MUSICIP PUID",                 "MUSICIP_PUID"),
-  };
-  const size_t txxxFrameTranslationSize = sizeof(txxxFrameTranslation) / sizeof(txxxFrameTranslation[0]);
-
-  // list of deprecated frames and their successors
-  const std::pair<const char *, const char *> deprecatedFrames[] = {
-    std::make_pair("TRDA", "TDRC"), // 2.3 -> 2.4 (http://en.wikipedia.org/wiki/ID3)
-    std::make_pair("TDAT", "TDRC"), // 2.3 -> 2.4
-    std::make_pair("TYER", "TDRC"), // 2.3 -> 2.4
-    std::make_pair("TIME", "TDRC"), // 2.3 -> 2.4
-  };
-  const size_t deprecatedFramesSize = sizeof(deprecatedFrames) / sizeof(deprecatedFrames[0]);
-}  // namespace
-
-String Frame::frameIDToKey(const ByteVector &id)
-{
-  ByteVector id24 = id;
-  for(size_t i = 0; i < deprecatedFramesSize; ++i) {
-    if(id24 == deprecatedFrames[i].first) {
-      id24 = deprecatedFrames[i].second;
-      break;
-    }
-  }
-  for(size_t i = 0; i < frameTranslationSize; ++i) {
-    if(id24 == frameTranslation[i].first)
-      return frameTranslation[i].second;
-  }
-  return String();
-}
-
-ByteVector Frame::keyToFrameID(const String &s)
-{
-  const String key = s.upper();
-  for(size_t i = 0; i < frameTranslationSize; ++i) {
-    if(key == frameTranslation[i].second)
-      return frameTranslation[i].first;
-  }
-  return ByteVector();
-}
-
-String Frame::txxxToKey(const String &description)
-{
-  const String d = description.upper();
-  for(size_t i = 0; i < txxxFrameTranslationSize; ++i) {
-    if(d == txxxFrameTranslation[i].first)
-      return txxxFrameTranslation[i].second;
-  }
-  return d;
-}
-
-String Frame::keyToTXXX(const String &s)
-{
-  const String key = s.upper();
-  for(size_t i = 0; i < txxxFrameTranslationSize; ++i) {
-    if(key == txxxFrameTranslation[i].second)
-      return txxxFrameTranslation[i].first;
-  }
-  return s;
-}
-
 PropertyMap Frame::asProperties() const
 {
   if(dynamic_cast< const UnknownFrame *>(this)) {
     PropertyMap m;
-    m.unsupportedData().append("UNKNOWN/" + frameID());
+    m.addUnsupportedData("UNKNOWN/" + frameID());
     return m;
   }
   const ByteVector &id = frameID();
-  // workaround until this function is virtual
-  if(id == "TXXX")
-    return dynamic_cast< const UserTextIdentificationFrame* >(this)->asProperties();
-  // Apple proprietary WFED (Podcast URL), MVNM (Movement Name), MVIN (Movement Number), GRP1 (Grouping) are in fact text frames.
-  if(id[0] == 'T' || id == "WFED" || id == "MVNM" || id == "MVIN" || id == "GRP1")
-    return dynamic_cast< const TextIdentificationFrame* >(this)->asProperties();
-  if(id == "WXXX")
-    return dynamic_cast< const UserUrlLinkFrame* >(this)->asProperties();
-  if(id[0] == 'W')
-    return dynamic_cast< const UrlLinkFrame* >(this)->asProperties();
-  if(id == "COMM")
-    return dynamic_cast< const CommentsFrame* >(this)->asProperties();
-  if(id == "USLT")
-    return dynamic_cast< const UnsynchronizedLyricsFrame* >(this)->asProperties();
-  if(id == "UFID")
-    return dynamic_cast< const UniqueFileIdentifierFrame* >(this)->asProperties();
-  if(id == "PCST")
-    return dynamic_cast< const PodcastFrame* >(this)->asProperties();
   PropertyMap m;
-  m.unsupportedData().append(id);
+  m.addUnsupportedData(id);
   return m;
 }
 
@@ -507,13 +386,13 @@ void Frame::splitProperties(const PropertyMap &original, PropertyMap &singleFram
   singleFrameProperties.clear();
   tiplProperties.clear();
   tmclProperties.clear();
-  for(PropertyMap::ConstIterator it = original.begin(); it != original.end(); ++it) {
-    if(TextIdentificationFrame::involvedPeopleMap().contains(it->first))
-      tiplProperties.insert(it->first, it->second);
-    else if(it->first.startsWith(TextIdentificationFrame::instrumentPrefix))
-      tmclProperties.insert(it->first, it->second);
+  for(const auto &[person, tag] : original) {
+    if(TextIdentificationFrame::involvedPeopleMap().contains(person))
+      tiplProperties.insert(person, tag);
+    else if(person.startsWith(TextIdentificationFrame::instrumentPrefix))
+      tmclProperties.insert(person, tag);
     else
-      singleFrameProperties.insert(it->first, it->second);
+      singleFrameProperties.insert(person, tag);
   }
 }
 
@@ -524,47 +403,29 @@ void Frame::splitProperties(const PropertyMap &original, PropertyMap &singleFram
 class Frame::Header::HeaderPrivate
 {
 public:
-  HeaderPrivate() :
-    frameSize(0),
-    version(4),
-    tagAlterPreservation(false),
-    fileAlterPreservation(false),
-    readOnly(false),
-    groupingIdentity(false),
-    compression(false),
-    encryption(false),
-    unsynchronisation(false),
-    dataLengthIndicator(false)
-    {}
-
   ByteVector frameID;
-  unsigned int frameSize;
-  unsigned int version;
+  unsigned int frameSize { 0 };
+  unsigned int version { 4 };
 
   // flags
 
-  bool tagAlterPreservation;
-  bool fileAlterPreservation;
-  bool readOnly;
-  bool groupingIdentity;
-  bool compression;
-  bool encryption;
-  bool unsynchronisation;
-  bool dataLengthIndicator;
+  bool tagAlterPreservation { false };
+  bool fileAlterPreservation { false };
+  bool readOnly { false };
+  bool groupingIdentity { false };
+  bool compression { false };
+  bool encryption { false };
+  bool unsynchronisation { false };
+  bool dataLengthIndicator { false };
 };
 
 ////////////////////////////////////////////////////////////////////////////////
-// static members (Frame::Header)
+// public members (Frame::Header)
 ////////////////////////////////////////////////////////////////////////////////
 
-unsigned int Frame::Header::size()
+unsigned int Frame::Header::size() const
 {
-  return size(4);
-}
-
-unsigned int Frame::Header::size(unsigned int version)
-{
-  switch(version) {
+  switch(d->version) {
   case 0:
   case 1:
   case 2:
@@ -576,31 +437,13 @@ unsigned int Frame::Header::size(unsigned int version)
   }
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// public members (Frame::Header)
-////////////////////////////////////////////////////////////////////////////////
-
-Frame::Header::Header(const ByteVector &data, bool synchSafeInts) :
-  d(new HeaderPrivate())
-{
-  setData(data, synchSafeInts);
-}
-
 Frame::Header::Header(const ByteVector &data, unsigned int version) :
-  d(new HeaderPrivate())
+  d(std::make_unique<HeaderPrivate>())
 {
   setData(data, version);
 }
 
-Frame::Header::~Header()
-{
-  delete d;
-}
-
-void Frame::Header::setData(const ByteVector &data, bool synchSafeInts)
-{
-  setData(data, static_cast<unsigned int>(synchSafeInts ? 4 : 3));
-}
+Frame::Header::~Header() = default;
 
 void Frame::Header::setData(const ByteVector &data, unsigned int version)
 {
@@ -798,11 +641,6 @@ bool Frame::Header::encryption() const
   return d->encryption;
 }
 
-bool Frame::Header::unsycronisation() const
-{
-  return unsynchronisation();
-}
-
 bool Frame::Header::unsynchronisation() const
 {
   return d->unsynchronisation;
@@ -815,7 +653,7 @@ bool Frame::Header::dataLengthIndicator() const
 
 ByteVector Frame::Header::render() const
 {
-  ByteVector flags(2, char(0)); // just blank for the moment
+  ByteVector flags(2, static_cast<char>(0)); // just blank for the moment
 
   ByteVector v = d->frameID +
     (d->version == 3
@@ -825,8 +663,3 @@ ByteVector Frame::Header::render() const
 
   return v;
 }
-
-bool Frame::Header::frameAlterPreservation() const
-{
-  return fileAlterPreservation();
-}
