@@ -23,76 +23,71 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tbytevector.h>
-#include <tstring.h>
-#include <tdebug.h>
-#include <tpropertymap.h>
-
-#include <xiphcomment.h>
 #include "oggflacfile.h"
 
+#include "tdebug.h"
+#include "tpropertymap.h"
+#include "tagutils.h"
+
 using namespace TagLib;
 using TagLib::FLAC::Properties;
 
 class Ogg::FLAC::File::FilePrivate
 {
 public:
-  FilePrivate() :
-    comment(0),
-    properties(0),
-    streamStart(0),
-    streamLength(0),
-    scanned(false),
-    hasXiphComment(false),
-    commentPacket(0) {}
+  std::unique_ptr<Ogg::XiphComment> comment;
 
-  ~FilePrivate()
-  {
-    delete comment;
-    delete properties;
-  }
-
-  Ogg::XiphComment *comment;
-
-  Properties *properties;
+  std::unique_ptr<Properties> properties;
   ByteVector streamInfoData;
   ByteVector xiphCommentData;
-  long streamStart;
-  long streamLength;
-  bool scanned;
+  offset_t streamStart { 0 };
+  offset_t streamLength { 0 };
+  bool scanned { false };
 
-  bool hasXiphComment;
-  int commentPacket;
+  bool hasXiphComment { false };
+  int commentPacket { 0 };
+  int lastHeaderPacket { 0 };
 };
 
+////////////////////////////////////////////////////////////////////////////////
+// static members
+////////////////////////////////////////////////////////////////////////////////
+
+bool Ogg::FLAC::File::isSupported(IOStream *stream)
+{
+  // An Ogg FLAC file has IDs "OggS" and "fLaC" somewhere.
+
+  const ByteVector buffer = Utils::readHeader(stream, bufferSize(), false);
+  return buffer.find("OggS") >= 0 && buffer.find("fLaC") >= 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
 Ogg::FLAC::File::File(FileName file, bool readProperties,
-                      Properties::ReadStyle propertiesStyle) : Ogg::File(file)
+                      Properties::ReadStyle propertiesStyle) :
+  Ogg::File(file),
+  d(std::make_unique<FilePrivate>())
 {
-  d = new FilePrivate;
   if(isOpen())
     read(readProperties, propertiesStyle);
 }
 
 Ogg::FLAC::File::File(IOStream *stream, bool readProperties,
-                      Properties::ReadStyle propertiesStyle) : Ogg::File(stream)
+                      Properties::ReadStyle propertiesStyle) :
+  Ogg::File(stream),
+  d(std::make_unique<FilePrivate>())
 {
-  d = new FilePrivate;
   if(isOpen())
     read(readProperties, propertiesStyle);
 }
 
-Ogg::FLAC::File::~File()
-{
-  delete d;
-}
+Ogg::FLAC::File::~File() = default;
 
 Ogg::XiphComment *Ogg::FLAC::File::tag() const
 {
-  return d->comment;
+  return d->comment.get();
 }
 
 PropertyMap Ogg::FLAC::File::properties() const
@@ -103,16 +98,20 @@ PropertyMap Ogg::FLAC::File::properties() const
 PropertyMap Ogg::FLAC::File::setProperties(const PropertyMap &properties)
 {
   return d->comment->setProperties(properties);
-}  
+}
 
 Properties *Ogg::FLAC::File::audioProperties() const
 {
-  return d->properties;
+  return d->properties.get();
 }
 
-
 bool Ogg::FLAC::File::save()
 {
+  if (d->commentPacket == 0) {
+    debug("Ogg::FLAC::File::save() -- Invalid, file has no Vorbis comment metadata block.");
+    return false;
+  }
+
   d->xiphCommentData = d->comment->render(false);
 
   // Create FLAC metadata-block:
@@ -120,11 +119,28 @@ bool Ogg::FLAC::File::save()
   // Put the size in the first 32 bit (I assume no more than 24 bit are used)
 
   ByteVector v = ByteVector::fromUInt(d->xiphCommentData.size());
+  if(v[0] != 0) {
+    // Block size uses more than 24 bits, try again with pictures removed.
+    d->comment->removeAllPictures();
+    d->xiphCommentData = d->comment->render(false);
+    v = ByteVector::fromUInt(d->xiphCommentData.size());
+    if(v[0] != 0) {
+      debug("Ogg::FLAC::File::save() -- Invalid, metadata block is too large.");
+      return false;
+    }
+    debug("Ogg::FLAC::File::save() -- Metadata block is too large, pictures removed.");
+  }
 
   // Set the type of the metadata-block to be a Xiph / Vorbis comment
 
   v[0] = 4;
 
+  // If the comment block is the last one, set the corresponding bit
+
+  if (d->commentPacket == d->lastHeaderPacket) {
+    v[0] = static_cast<char>(0x84);
+  }
+
   // Append the comment-data after the 32 bit header
 
   v.append(d->xiphCommentData);
@@ -170,13 +186,12 @@ void Ogg::FLAC::File::read(bool readProperties, Properties::ReadStyle properties
 
 
   if(d->hasXiphComment)
-    d->comment = new Ogg::XiphComment(xiphCommentData());
+    d->comment = std::make_unique<Ogg::XiphComment>(xiphCommentData());
   else
-    d->comment = new Ogg::XiphComment;
-
+    d->comment = std::make_unique<Ogg::XiphComment>();
 
   if(readProperties)
-    d->properties = new Properties(streamInfoData(), streamLength(), propertiesStyle);
+    d->properties = std::make_unique<Properties>(streamInfoData(), streamLength(), propertiesStyle);
 }
 
 ByteVector Ogg::FLAC::File::streamInfoData()
@@ -191,7 +206,7 @@ ByteVector Ogg::FLAC::File::xiphCommentData()
   return d->xiphCommentData;
 }
 
-long Ogg::FLAC::File::streamLength()
+offset_t Ogg::FLAC::File::streamLength()
 {
   scan();
   return d->streamLength;
@@ -208,32 +223,43 @@ void Ogg::FLAC::File::scan()
     return;
 
   int ipacket = 0;
-  long overhead = 0;
+  offset_t overhead = 0;
 
   ByteVector metadataHeader = packet(ipacket);
-  if(metadataHeader.isNull())
+  if(metadataHeader.isEmpty())
     return;
 
-  ByteVector header;
-
-  if (!metadataHeader.startsWith("fLaC"))  {
+  if(!metadataHeader.startsWith("fLaC"))  {
     // FLAC 1.1.2+
-    if (metadataHeader.mid(1,4) != "FLAC") return;
+    // See https://xiph.org/flac/ogg_mapping.html for the header specification.
+    if(metadataHeader.size() < 13)
+      return;
 
-    if (metadataHeader[5] != 1) return; // not version 1
+    if(metadataHeader[0] != 0x7f)
+      return;
+
+    if(metadataHeader.mid(1, 4) != "FLAC")
+      return;
+
+    if(metadataHeader[5] != 1 && metadataHeader[6] != 0)
+      return; // not version 1.0
+
+    if(metadataHeader.mid(9, 4) != "fLaC")
+      return;
 
     metadataHeader = metadataHeader.mid(13);
   }
   else {
     // FLAC 1.1.0 & 1.1.1
     metadataHeader = packet(++ipacket);
-
-    if(metadataHeader.isNull())
-      return;
-
   }
 
-  header = metadataHeader.mid(0,4);
+  ByteVector header = metadataHeader.mid(0, 4);
+  if(header.size() != 4) {
+    debug("Ogg::FLAC::File::scan() -- Invalid Ogg/FLAC metadata header");
+    return;
+  }
+
   // Header format (from spec):
   // <1> Last-metadata-block flag
   // <7> BLOCK_TYPE
@@ -246,7 +272,7 @@ void Ogg::FLAC::File::scan()
 
   char blockType = header[0] & 0x7f;
   bool lastBlock = (header[0] & 0x80) != 0;
-  uint length = header.toUInt(1, 3, true);
+  unsigned int length = header.toUInt(1, 3, true);
   overhead += length;
 
   // Sanity: First block should be the stream_info metadata
@@ -262,11 +288,18 @@ void Ogg::FLAC::File::scan()
 
   while(!lastBlock) {
     metadataHeader = packet(++ipacket);
-
-    if(metadataHeader.isNull())
-      return;
-
     header = metadataHeader.mid(0, 4);
+    if(header.size() != 4) {
+      debug("Ogg::FLAC::File::scan() -- Invalid Ogg/FLAC metadata header");
+      return;
+    }
+
+    if(((header[0] & 0xff) == 0xff) && ((header[1] & 0xff) == 0xf8)) {
+      ipacket--;
+      debug("Ogg::FLAC::File::scan() -- Found frame sync marker, possibly missing last block marker");
+      break;
+    }
+
     blockType = header[0] & 0x7f;
     lastBlock = (header[0] & 0x80) != 0;
     length = header.toUInt(1, 3, true);
@@ -275,16 +308,32 @@ void Ogg::FLAC::File::scan()
     if(blockType == 1) {
       // debug("Ogg::FLAC::File::scan() -- Padding found");
     }
+    else if(blockType == 2) {
+      // debug("Ogg::FLAC::File::scan() -- Application block found");
+    }
+    else if(blockType == 3) {
+      // debug("Ogg::FLAC::File::scan() -- Seek table found");
+    }
     else if(blockType == 4) {
       // debug("Ogg::FLAC::File::scan() -- Vorbis-comments found");
       d->xiphCommentData = metadataHeader.mid(4, length);
       d->hasXiphComment = true;
       d->commentPacket = ipacket;
     }
-    else if(blockType > 5) {
+    else if(blockType == 5) {
+      // debug("Ogg::FLAC::File::scan() -- Cuesheet found");
+    }
+    else if(blockType == 6) {
+      // debug("Ogg::FLAC::File::scan() -- Picture found");
+    }
+    else if(blockType == 127) {
+      debug("Ogg::FLAC::File::scan() -- Forbidden block type found");
+    }
+    else if(blockType > 6) {
       debug("Ogg::FLAC::File::scan() -- Unknown metadata block");
     }
   }
+  d->lastHeaderPacket = ipacket;
 
   // End of metadata, now comes the datastream
   d->streamStart = overhead;
